@@ -18,7 +18,6 @@
 #include <girara/utils.h>
 #include <glib/gi18n.h>
 
-#define GLOBAL_RC "/etc/zathurarc"
 #define ZATHURA_RC "zathurarc"
 
 static void cb_jumplist_change(girara_session_t* session, const char* UNUSED(name), girara_setting_type_t UNUSED(type),
@@ -28,25 +27,19 @@ static void cb_jumplist_change(girara_session_t* session, const char* UNUSED(nam
   g_return_if_fail(session->global.data != NULL);
   zathura_t* zathura = session->global.data;
 
-  const int* ivalue          = value;
-  zathura->jumplist.max_size = MAX(0, *ivalue);
-
-  if (zathura->jumplist.list != NULL && zathura->jumplist.size != 0) {
-    zathura_jumplist_trim(zathura);
-  }
+  const int* ivalue = value;
+  zathura_jumplist_set_max_size(zathura, MAX(0, *ivalue));
 }
 
-static void
-cb_color_change(girara_session_t* session, const char* name,
-                girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
-{
+static void cb_color_change(girara_session_t* session, const char* name, girara_setting_type_t UNUSED(type),
+                            const void* value, void* UNUSED(data)) {
   g_return_if_fail(value != NULL);
   g_return_if_fail(session != NULL);
   g_return_if_fail(session->global.data != NULL);
   g_return_if_fail(name != NULL);
   zathura_t* zathura = session->global.data;
 
-  const char* string_value = (const char*) value;
+  const char* string_value = (const char*)value;
   if (g_strcmp0(name, "highlight-color") == 0) {
     parse_color(&zathura->ui.colors.highlight_color, string_value);
   } else if (g_strcmp0(name, "highlight-fg") == 0) {
@@ -76,10 +69,8 @@ cb_color_change(girara_session_t* session, const char* name,
   render_all(zathura);
 }
 
-static void
-cb_nohlsearch_changed(girara_session_t* session, const char* UNUSED(name),
-                      girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
-{
+static void cb_nohlsearch_changed(girara_session_t* session, const char* UNUSED(name),
+                                  girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data)) {
   g_return_if_fail(value != NULL);
   g_return_if_fail(session != NULL);
   g_return_if_fail(session->global.data != NULL);
@@ -90,23 +81,18 @@ cb_nohlsearch_changed(girara_session_t* session, const char* UNUSED(name),
   render_all(zathura);
 }
 
-static void
-cb_doubleclick_changed(girara_session_t* session, const char* UNUSED(name),
-                      girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
-{
+static void cb_doubleclick_changed(girara_session_t* session, const char* UNUSED(name),
+                                   girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data)) {
   g_return_if_fail(value != NULL);
   g_return_if_fail(session != NULL);
   g_return_if_fail(session->global.data != NULL);
   zathura_t* zathura = session->global.data;
 
-  zathura->global.double_click_follow = *(const bool*) value;
+  zathura->global.double_click_follow = *(const bool*)value;
 }
 
-static void
-cb_global_modifiers_changed(girara_session_t* session, const char* name,
-                            girara_setting_type_t UNUSED(type), const void* value,
-                            void* UNUSED(data))
-{
+static void cb_global_modifiers_changed(girara_session_t* session, const char* name, girara_setting_type_t UNUSED(type),
+                                        const void* value, void* UNUSED(data)) {
   g_return_if_fail(value != NULL);
   g_return_if_fail(session != NULL);
   g_return_if_fail(session->global.data != NULL);
@@ -125,7 +111,7 @@ cb_global_modifiers_changed(girara_session_t* session, const char* name,
   const char* modifier = value;
   if (g_strcmp0(modifier, "shift") == 0) {
     *p = GDK_SHIFT_MASK;
-  } else if (g_strcmp0(modifier, "ctrl") == 0)  {
+  } else if (g_strcmp0(modifier, "ctrl") == 0) {
     *p = GDK_CONTROL_MASK;
   } else if (g_strcmp0(modifier, "alt") == 0) {
     *p = GDK_MOD1_MASK;
@@ -134,42 +120,19 @@ cb_global_modifiers_changed(girara_session_t* session, const char* name,
   }
 }
 
-static void
-cb_incsearch_changed(girara_session_t* session, const char* UNUSED(name),
-                     girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
-{
+static void cb_incsearch_changed(girara_session_t* session, const char* UNUSED(name),
+                                 girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data)) {
   g_return_if_fail(value != NULL);
   g_return_if_fail(session != NULL);
   g_return_if_fail(session->global.data != NULL);
 
-  bool inc_search = *(const bool*) value;
-  girara_special_command_add(session, '/', cmd_search, inc_search, FORWARD,  NULL);
+  bool inc_search = *(const bool*)value;
+  girara_special_command_add(session, '/', cmd_search, inc_search, FORWARD, NULL);
   girara_special_command_add(session, '?', cmd_search, inc_search, BACKWARD, NULL);
 }
 
-static void
-cb_sandbox_changed(girara_session_t* session, const char* name,
-                   girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
-{
-  g_return_if_fail(value != NULL);
-  g_return_if_fail(session != NULL);
-  g_return_if_fail(session->global.data != NULL);
-  zathura_t* zathura = session->global.data;
-
-  const char* sandbox = value;
-  if (g_strcmp0(sandbox, "none") == 0) {
-    zathura->global.sandbox = ZATHURA_SANDBOX_NONE;
-  } else if (g_strcmp0(sandbox, "strict") == 0) {
-    zathura->global.sandbox = ZATHURA_SANDBOX_STRICT;
-  } else {
-    girara_error("Invalid %s option: '%s'", name, sandbox);
-  }
-}
-
-static void
-cb_window_statbusbar_changed(girara_session_t* session, const char* name,
-                             girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data))
-{
+static void cb_window_statbusbar_changed(girara_session_t* session, const char* name,
+                                         girara_setting_type_t UNUSED(type), const void* value, void* UNUSED(data)) {
   g_return_if_fail(value != NULL);
   g_return_if_fail(session != NULL);
   g_return_if_fail(session->global.data != NULL);
@@ -192,7 +155,7 @@ static void cb_show_signature_info(girara_session_t* session, const char* UNUSED
   g_return_if_fail(session->global.data != NULL);
   zathura_t* zathura = session->global.data;
 
-  if (zathura->document == NULL) {
+  if (zathura_has_document(zathura) == false) {
     return;
   }
 
@@ -224,11 +187,7 @@ void config_load_default(zathura_t* zathura) {
 #define PRESENTATION zathura->modes.presentation
 
   const girara_mode_t all_modes[] = {
-    NORMAL,
-    INSERT,
-    INDEX,
-    FULLSCREEN,
-    PRESENTATION
+      NORMAL, INSERT, INDEX, FULLSCREEN, PRESENTATION,
   };
 
   /* Set default mode */
@@ -326,7 +285,7 @@ void config_load_default(zathura_t* zathura) {
   bool_value = false;
   girara_setting_add(gsession, "open-first-page",        &bool_value,  BOOLEAN, false, _("Always open on first page"), NULL, NULL);
   bool_value = false;
-  girara_setting_add(gsession, "nohlsearch",             &bool_value,  BOOLEAN, false, _("Highlight search results"), cb_nohlsearch_changed, NULL);
+  girara_setting_add(gsession, "nohlsearch",             &bool_value,  BOOLEAN, false, _("Disable highlighting of search results"), cb_nohlsearch_changed, NULL);
   bool_value = true;
   girara_setting_add(gsession, "double-click-follow",    &bool_value,  BOOLEAN, false, _("Double click to follow links"), cb_doubleclick_changed, NULL);
 #define INCREMENTAL_SEARCH false
@@ -361,9 +320,6 @@ void config_load_default(zathura_t* zathura) {
   girara_setting_add(gsession, "selection-clipboard",    "primary",    STRING,  false, _("The clipboard into which mouse-selected data will be written"), NULL, NULL);
   bool_value = true;
   girara_setting_add(gsession, "selection-notification", &bool_value,  BOOLEAN, false, _("Enable notification after selecting text"), NULL, NULL);
-  /* default to no sandbox */
-  const char* string_value = "none";
-  girara_setting_add(gsession, "sandbox",                string_value, STRING, true,   _("Sandbox level"), cb_sandbox_changed, NULL);
   bool_value = false;
   girara_setting_add(gsession, "show-signature-information", &bool_value, BOOLEAN, false,
                      _("Disable additional information for signatures embedded in the document."),
@@ -386,7 +342,9 @@ void config_load_default(zathura_t* zathura) {
   girara_shortcut_add(gsession, 0, GDK_KEY_f, NULL, sc_follow, (mode), 0, NULL);                                       \
                                                                                                                        \
   girara_shortcut_add(gsession, 0, 0, "gg", sc_goto, (mode), TOP, NULL);                                               \
+  girara_shortcut_add(gsession, 0, GDK_KEY_Home, NULL, sc_goto, (mode), TOP, NULL);                                    \
   girara_shortcut_add(gsession, 0, 0, "G", sc_goto, (mode), BOTTOM, NULL);                                             \
+  girara_shortcut_add(gsession, 0, GDK_KEY_End, NULL, sc_goto, (mode), BOTTOM, NULL);                                  \
                                                                                                                        \
   girara_shortcut_add(gsession, 0, GDK_KEY_m, NULL, sc_mark_add, (mode), 0, NULL);                                     \
   girara_shortcut_add(gsession, 0, GDK_KEY_apostrophe, NULL, sc_mark_evaluate, (mode), 0, NULL);                       \
@@ -397,6 +355,8 @@ void config_load_default(zathura_t* zathura) {
   girara_shortcut_add(gsession, GDK_MOD1_MASK, GDK_KEY_Left, NULL, sc_navigate, (mode), PREVIOUS, NULL);               \
   girara_shortcut_add(gsession, 0, GDK_KEY_Page_Down, NULL, sc_navigate, (mode), NEXT, NULL);                          \
   girara_shortcut_add(gsession, 0, GDK_KEY_Page_Up, NULL, sc_navigate, (mode), PREVIOUS, NULL);                        \
+  girara_shortcut_add(gsession, 0, GDK_KEY_Forward, NULL, sc_navigate, (mode), NEXT, NULL);                            \
+  girara_shortcut_add(gsession, 0, GDK_KEY_Back, NULL, sc_navigate, (mode), PREVIOUS, NULL);                           \
                                                                                                                        \
   girara_shortcut_add(gsession, GDK_CONTROL_MASK, GDK_KEY_p, NULL, sc_print, (mode), 0, NULL);                         \
                                                                                                                        \
@@ -536,11 +496,13 @@ void config_load_default(zathura_t* zathura) {
   girara_shortcut_add(gsession, 0,              GDK_KEY_Down,      NULL, sc_navigate,            PRESENTATION, NEXT,         NULL);
   girara_shortcut_add(gsession, 0,              GDK_KEY_Right,     NULL, sc_navigate,            PRESENTATION, NEXT,         NULL);
   girara_shortcut_add(gsession, 0,              GDK_KEY_Page_Down, NULL, sc_navigate,            PRESENTATION, NEXT,         NULL);
+  girara_shortcut_add(gsession, 0,              GDK_KEY_Forward,   NULL, sc_navigate,            PRESENTATION, NEXT,         NULL);
   girara_shortcut_add(gsession, 0,              GDK_KEY_space,     NULL, sc_navigate,            PRESENTATION, NEXT,         NULL);
   girara_shortcut_add(gsession, 0,              GDK_KEY_K,         NULL, sc_navigate,            PRESENTATION, PREVIOUS,     NULL);
   girara_shortcut_add(gsession, 0,              GDK_KEY_Left,      NULL, sc_navigate,            PRESENTATION, PREVIOUS,     NULL);
   girara_shortcut_add(gsession, 0,              GDK_KEY_Up,        NULL, sc_navigate,            PRESENTATION, PREVIOUS,     NULL);
   girara_shortcut_add(gsession, 0,              GDK_KEY_Page_Up,   NULL, sc_navigate,            PRESENTATION, PREVIOUS,     NULL);
+  girara_shortcut_add(gsession, 0,              GDK_KEY_Back,      NULL, sc_navigate,            PRESENTATION, PREVIOUS,     NULL);
   girara_shortcut_add(gsession, GDK_SHIFT_MASK, GDK_KEY_space,     NULL, sc_navigate,            PRESENTATION, PREVIOUS,     NULL);
   girara_shortcut_add(gsession, 0,              GDK_KEY_BackSpace, NULL, sc_navigate,            PRESENTATION, PREVIOUS,     NULL);
 
@@ -570,7 +532,9 @@ void config_load_default(zathura_t* zathura) {
   /* define default inputbar commands */
   girara_inputbar_command_add(gsession, "bmark",      NULL,   cmd_bookmark_create, NULL,         _("Add a bookmark"));
   girara_inputbar_command_add(gsession, "bdelete",    NULL,   cmd_bookmark_delete, cc_bookmarks, _("Delete a bookmark"));
-  girara_inputbar_command_add(gsession, "blist",      NULL,   cmd_bookmark_open,   cc_bookmarks, _("List all bookmarks"));
+  girara_inputbar_command_add(gsession, "blist",      NULL,   cmd_bookmark_list,   NULL,         _("List all bookmarks"));
+  girara_inputbar_command_add(gsession, "bjump",      NULL,   cmd_bookmark_open,   cc_bookmarks, _("Jump to bookmark"));
+  girara_inputbar_command_add(gsession, "jumplist",   NULL,   cmd_jumplist_list,   NULL,         _("Show recent jumps in jumplist"));
   girara_inputbar_command_add(gsession, "close",      NULL,   cmd_close,           NULL,         _("Close current file"));
   girara_inputbar_command_add(gsession, "info",       NULL,   cmd_info,            NULL,         _("Show file information"));
   girara_inputbar_command_add(gsession, "exec",       NULL,   cmd_exec,            NULL,         _("Execute a command"));
@@ -587,7 +551,7 @@ void config_load_default(zathura_t* zathura) {
   girara_inputbar_command_add(gsession, "offset",     NULL,   cmd_offset,          NULL,         _("Set page offset"));
   girara_inputbar_command_add(gsession, "mark",       NULL,   cmd_marks_add,       NULL,         _("Mark current location within the document"));
   girara_inputbar_command_add(gsession, "delmarks",   "delm", cmd_marks_delete,    NULL,         _("Delete the specified marks"));
-  girara_inputbar_command_add(gsession, "nohlsearch", "nohl", cmd_nohlsearch,      NULL,         _("Don't highlight current search results"));
+  girara_inputbar_command_add(gsession, "nohlsearch", "nohl", cmd_nohlsearch,      NULL,         _("Remove highlights of current search results"));
   girara_inputbar_command_add(gsession, "hlsearch",   NULL,   cmd_hlsearch,        NULL,         _("Highlight current search results"));
   girara_inputbar_command_add(gsession, "version",    NULL,   cmd_version,         NULL,         _("Show version information"));
   girara_inputbar_command_add(gsession, "source",     NULL,   cmd_source,          NULL,         _("Source config file"));
@@ -666,23 +630,21 @@ void config_load_default(zathura_t* zathura) {
   /* clang-format on */
 }
 
-void
-config_load_files(zathura_t* zathura)
-{
+void config_load_files(zathura_t* zathura) {
   /* load global configuration files */
-  char* config_path = girara_get_xdg_path(XDG_CONFIG_DIRS);
+  char* config_path          = girara_get_xdg_path(XDG_CONFIG_DIRS);
   girara_list_t* config_dirs = girara_split_path_array(config_path);
-  ssize_t size = girara_list_size(config_dirs) - 1;
+  ssize_t size               = girara_list_size(config_dirs) - 1;
   for (; size >= 0; --size) {
     const char* dir = girara_list_nth(config_dirs, size);
-    char* file = g_build_filename(dir, ZATHURA_RC, NULL);
+    char* file      = g_build_filename(dir, ZATHURA_RC, NULL);
     girara_config_parse(zathura->ui.session, file);
     g_free(file);
   }
   girara_list_free(config_dirs);
   g_free(config_path);
 
-  girara_config_parse(zathura->ui.session, GLOBAL_RC);
+  girara_config_parse(zathura->ui.session, SYSCONFDIR "/" ZATHURA_RC);
 
   /* load local configuration files */
   char* configuration_file = g_build_filename(zathura->config.config_dir, ZATHURA_RC, NULL);
