@@ -13,9 +13,7 @@ struct zathura_content_type_context_s {
   magic_t magic;
 };
 
-zathura_content_type_context_t*
-zathura_content_type_new(void)
-{
+zathura_content_type_context_t* zathura_content_type_new(void) {
   zathura_content_type_context_t* context = g_try_malloc0(sizeof(zathura_content_type_context_t));
   if (context == NULL) {
     return NULL;
@@ -41,9 +39,7 @@ zathura_content_type_new(void)
   return context;
 }
 
-void
-zathura_content_type_free(zathura_content_type_context_t* context)
-{
+void zathura_content_type_free(zathura_content_type_context_t* context) {
   if (context != NULL && context->magic != NULL) {
     magic_close(context->magic);
   }
@@ -54,9 +50,7 @@ zathura_content_type_free(zathura_content_type_context_t* context)
 /** Read a most GT_MAX_READ bytes before falling back to file. */
 static const size_t GT_MAX_READ = 1 << 16;
 
-static char*
-guess_type_magic(zathura_content_type_context_t* context, const char* path)
-{
+static char* guess_type_magic(zathura_content_type_context_t* context, const char* path) {
   if (context == NULL || context->magic == NULL) {
     return NULL;
   }
@@ -81,11 +75,9 @@ guess_type_magic(zathura_content_type_context_t* context, const char* path)
   return content_type;
 }
 
-static char*
-guess_type_glib(const char* path)
-{
-  gboolean uncertain    = FALSE;
-  char*    content_type = g_content_type_guess(path, NULL, 0, &uncertain);
+static char* guess_type_glib(const char* path) {
+  gboolean uncertain = FALSE;
+  char* content_type = g_content_type_guess(path, NULL, 0, &uncertain);
   if (content_type == NULL) {
     girara_debug("g_content_type failed\n");
   } else {
@@ -102,7 +94,7 @@ guess_type_glib(const char* path)
   }
 
   guchar* content = NULL;
-  size_t  length  = 0;
+  size_t length   = 0;
   while (uncertain == TRUE && length < GT_MAX_READ) {
     g_free(content_type);
     content_type = NULL;
@@ -133,16 +125,12 @@ guess_type_glib(const char* path)
   return NULL;
 }
 
-static int
-compare_content_types(const void* lhs, const void* rhs)
-{
+static int compare_content_types(const void* lhs, const void* rhs) {
   return g_strcmp0(lhs, rhs);
 }
 
-char*
-zathura_content_type_guess(zathura_content_type_context_t* context, const char* path,
-                           const girara_list_t* supported_content_types)
-{
+char* zathura_content_type_guess(zathura_content_type_context_t* context, const char* path,
+                                 const girara_list_t* supported_content_types) {
   /* try libmagic first */
   char* content_type = guess_type_magic(context, path);
   if (content_type != NULL) {
