@@ -16,27 +16,42 @@
 #include "adjustment.h"
 
 #ifdef WITH_SYNCTEX
-bool synctex_get_input_line_column(const char* filename, unsigned int page, int x, int y, char** input_file,
-                                   unsigned int* line, unsigned int* column) {
-  if (filename == NULL) {
-    return false;
+// Create scanner from given PDF file name.
+// Returns zathura->synctex.scanner. (May be NULL on error.)
+synctex_scanner_p synctex_make_scanner(zathura_t* zathura, const char* pdf_filename) {
+  if (zathura->synctex.scanner) {
+    return zathura->synctex.scanner;
   }
 
-  synctex_scanner_p scanner = synctex_scanner_new_with_output_file(filename, NULL, 1);
+  synctex_scanner_p scanner = synctex_scanner_new_with_output_file(pdf_filename, NULL, 1);
   if (scanner == NULL) {
     girara_debug("Failed to create synctex scanner.");
-    return false;
+    return NULL;
   }
 
   synctex_scanner_p temp = synctex_scanner_parse(scanner);
   if (temp == NULL) {
     girara_debug("Failed to parse synctex file.");
     synctex_scanner_free(scanner);
+    return NULL;
+  }
+
+  zathura->synctex.scanner = scanner;
+  return scanner;
+}
+
+bool synctex_get_input_line_column(zathura_t* zathura, const char* filename, unsigned int page, int x, int y,
+                                   char** input_file, unsigned int* line, unsigned int* column) {
+  if (filename == NULL) {
+    return false;
+  }
+
+  synctex_scanner_p scanner = synctex_make_scanner(zathura, filename);
+  if (!scanner) {
     return false;
   }
 
   bool ret = false;
-
   if (synctex_edit_query(scanner, page + 1u, x, y) > 0) {
     /* Assume that a backward search returns at most one result. */
     synctex_node_p node = synctex_scanner_next_result(scanner);
@@ -55,12 +70,10 @@ bool synctex_get_input_line_column(const char* filename, unsigned int page, int
     }
   }
 
-  synctex_scanner_free(scanner);
-
   return ret;
 }
 
-void synctex_edit(const char* editor, zathura_page_t* page, int x, int y) {
+void synctex_edit(zathura_t* zathura, const char* editor, zathura_page_t* page, int x, int y) {
   if (editor == NULL || page == NULL) {
     return;
   }
@@ -79,8 +92,8 @@ void synctex_edit(const char* editor, zathura_page_t* page, int x, int y) {
   unsigned int column = 0;
   char* input_file    = NULL;
 
-  if (synctex_get_input_line_column(filename, zathura_page_get_index(page), x, y, &input_file, &line, &column) ==
-          true &&
+  if (synctex_get_input_line_column(zathura, filename, zathura_page_get_index(page), x, y, &input_file, &line,
+                                    &column) == true &&
       input_file != NULL) {
     char* linestr   = g_strdup_printf("%d", line);
     char* columnstr = g_strdup_printf("%d", column);
@@ -124,8 +137,9 @@ void synctex_edit(const char* editor, zathura_page_t* page, int x, int y) {
   }
 }
 
-girara_list_t* synctex_rectangles_from_position(const char* filename, const char* input_file, int line, int column,
-                                                unsigned int* page, girara_list_t** secondary_rects) {
+girara_list_t* synctex_rectangles_from_position(zathura_t* zathura, const char* filename, const char* input_file,
+                                                int line, int column, unsigned int* page,
+                                                girara_list_t** secondary_rects) {
   if (filename == NULL || input_file == NULL || page == NULL) {
     return NULL;
   }
@@ -134,17 +148,9 @@ girara_list_t* synctex_rectangles_from_position(const char* filename, const char
   ++line;
   ++column;
 
-  synctex_scanner_p scanner = synctex_scanner_new_with_output_file(filename, NULL, 1);
-  if (scanner == NULL) {
-    girara_debug("Failed to create synctex scanner.");
-    return NULL;
-  }
-
-  synctex_scanner_p temp = synctex_scanner_parse(scanner);
-  if (temp == NULL) {
-    girara_debug("Failed to parse synctex file.");
-    synctex_scanner_free(scanner);
-    return NULL;
+  synctex_scanner_p scanner = synctex_make_scanner(zathura, filename);
+  if (!scanner) {
+    return false;
   }
 
   girara_list_t* hitlist     = girara_list_new2(g_free);
@@ -189,8 +195,6 @@ girara_list_t* synctex_rectangles_from_position(const char* filename, const char
     }
   }
 
-  synctex_scanner_free(scanner);
-
   if (secondary_rects != NULL) {
     *secondary_rects = other_rects;
   } else {
@@ -203,17 +207,18 @@ girara_list_t* synctex_rectangles_from_position(const char* filename, const char
   return hitlist_flat;
 }
 #else
-bool synctex_get_input_line_column(const char* UNUSED(filename), unsigned int UNUSED(page), int UNUSED(x),
-                                   int UNUSED(y), char** UNUSED(input_file), unsigned int* UNUSED(line),
+bool synctex_get_input_line_column(zathura_t* UNUSED(zathura), const char* UNUSED(filename), unsigned int UNUSED(page),
+                                   int UNUSED(x), int UNUSED(y), char** UNUSED(input_file), unsigned int* UNUSED(line),
                                    unsigned int* UNUSED(column)) {
   return false;
 }
 
-void synctex_edit(const char* UNUSED(editor), zathura_page_t* UNUSED(page), int UNUSED(x), int UNUSED(y)) {}
+void synctex_edit(zathura_t* UNUSED(zathura), const char* UNUSED(editor), zathura_page_t* UNUSED(page), int UNUSED(x),
+                  int UNUSED(y)) {}
 
-girara_list_t* synctex_rectangles_from_position(const char* UNUSED(filename), const char* UNUSED(input_file),
-                                                int UNUSED(line), int UNUSED(column), unsigned int* UNUSED(page),
-                                                girara_list_t** UNUSED(secondary_rects)) {
+girara_list_t* synctex_rectangles_from_position(zathura_t* UNUSED(zathura), const char* UNUSED(filename),
+                                                const char* UNUSED(input_file), int UNUSED(line), int UNUSED(column),
+                                                unsigned int* UNUSED(page), girara_list_t** UNUSED(secondary_rects)) {
   return NULL;
 }
 #endif
@@ -248,7 +253,8 @@ bool synctex_parse_input(const char* synctex, char** input_file, int* line, int*
 }
 
 void synctex_highlight_rects(zathura_t* zathura, unsigned int page, girara_list_t** rectangles) {
-  const unsigned int number_of_pages = zathura_document_get_number_of_pages(zathura->document);
+  zathura_document_t* document       = zathura_get_document(zathura);
+  const unsigned int number_of_pages = zathura_document_get_number_of_pages(document);
 
   for (unsigned int p = 0; p != number_of_pages; ++p) {
     GObject* widget = G_OBJECT(zathura->pages[p]);
@@ -274,20 +280,20 @@ void synctex_highlight_rects(zathura_t* zathura, unsigned int page, girara_list_
   /* compute the position of the center of the page */
   double pos_x = 0;
   double pos_y = 0;
-  page_number_to_position(zathura->document, page, 0.5, 0.5, &pos_x, &pos_y);
+  page_number_to_position(document, page, 0.5, 0.5, &pos_x, &pos_y);
 
   /* correction to center the current result                          */
   /* NOTE: rectangle is in viewport units, already scaled and rotated */
   unsigned int cell_height = 0;
   unsigned int cell_width  = 0;
-  zathura_document_get_cell_size(zathura->document, &cell_height, &cell_width);
+  zathura_document_get_cell_size(document, &cell_height, &cell_width);
 
   unsigned int doc_height = 0;
   unsigned int doc_width  = 0;
-  zathura_document_get_document_size(zathura->document, &doc_height, &doc_width);
+  zathura_document_get_document_size(document, &doc_height, &doc_width);
 
   /* Need to adjust rectangle to page scale and orientation */
-  zathura_page_t* doc_page  = zathura_document_get_page(zathura->document, page);
+  zathura_page_t* doc_page  = zathura_document_get_page(document, page);
   zathura_rectangle_t* rect = girara_list_nth(rect_list, 0);
   if (rect == NULL) {
     girara_debug("List of rectangles is broken. Jumping to page %u.", page);
@@ -330,11 +336,12 @@ bool synctex_view(zathura_t* zathura, const char* input_file, unsigned int line,
     return false;
   }
 
-  const unsigned int number_of_pages = zathura_document_get_number_of_pages(zathura->document);
+  zathura_document_t* document       = zathura_get_document(zathura);
+  const unsigned int number_of_pages = zathura_document_get_number_of_pages(document);
 
   unsigned int page              = 0;
   girara_list_t* secondary_rects = NULL;
-  girara_list_t* rectangles = synctex_rectangles_from_position(zathura_document_get_path(zathura->document), input_file,
+  girara_list_t* rectangles = synctex_rectangles_from_position(zathura, zathura_document_get_path(document), input_file,
                                                                line, column, &page, &secondary_rects);
 
   if (rectangles == NULL) {
